"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _Bool = _interopRequireDefault(require("./Bool"));

// Copyright 2017-2019 @polkadot/types authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
describe('Bool', () => {
  describe('decode', () => {
    // eslint-disable-next-line @typescript-eslint/ban-types
    const testDecode = (type, input, expected) => it("can decode from ".concat(type), () => {
      expect(new _Bool.default(input).toJSON()).toBe(expected);
    });

    testDecode('Bool', new _Bool.default(true), true);
    testDecode('Boolean', Boolean(true), true);
    testDecode('boolean', true, true);
    testDecode('number', 1, true);
    testDecode('Uint8Array', Uint8Array.from([1]), true);
  });
  describe('encode', () => {
    const testEncode = (to, expected, value) => it("can encode ".concat(to), () => {
      expect(new _Bool.default(value)[to]()).toEqual(expected);
    });

    testEncode('toJSON', true, true);
    testEncode('toHex', '0x01', true);
    testEncode('toString', 'true', true);
    testEncode('toU8a', Uint8Array.from([1]), true);
    testEncode('toU8a', Uint8Array.from([0]), false);
  });
  it('correctly encodes length', () => {
    expect(new _Bool.default(true).encodedLength).toEqual(1);
  });
  describe('utils', () => {
    it('compares agains a boolean', () => {
      expect(new _Bool.default(true).eq(true)).toBe(true);
    });
    it('compares agains a Bool', () => {
      expect(new _Bool.default(false).eq(new _Bool.default(false))).toBe(true);
    });
    it('has isTrue', () => {
      expect(new _Bool.default(true).isTrue).toBe(true);
    });
    it('has isFalse', () => {
      expect(new _Bool.default(true).isFalse).toBe(false);
    });
  });
});