import { AuthorityId } from '../../interfaces/consensus';
import { Consensus, Hash, PreRuntime, Seal, SealV0 } from '../../interfaces/runtime';
import Enum from '../../codec/Enum';
import Struct from '../../codec/Struct';
import Vec from '../../codec/Vec';
import Bytes from '../Bytes';
/**
 * @name DigestItem
 * @description
 * A [[Enum]] the specifies the specific item in the logs of a [[Digest]]
 */
export declare class DigestItem extends Enum {
    constructor(value: any);
    /**
     * @description Returns the item as a [[Vec<AuthorityId>]]
     */
    readonly asAuthoritiesChange: Vec<AuthorityId>;
    /**
     * @description Returns the item as a [[ChangesTrieRoot]]
     */
    readonly asChangesTrieRoot: Hash;
    /**
     * @desciption Retuns the item as a [[Consensus]]
     */
    readonly asConsensus: Consensus;
    /**
     * @description Returns the item as a [[Bytes]]
     */
    readonly asOther: Bytes;
    /**
     * @description Returns the item as a [[PreRuntime]]
     */
    readonly asPreRuntime: PreRuntime;
    /**
     * @description Returns the item as a [[Seal]]
     */
    readonly asSeal: Seal;
    /**
     * @description Returns the item as a [[SealV0]]
     */
    readonly asSealV0: SealV0;
    /**
     * @description Returns true on [[AuthoritiesChange]]
     */
    readonly isAuthoritiesChange: boolean;
    /**
     * @description Returns true on [[ChangesTrieRoot]]
     */
    readonly isChangesTrieRoot: boolean;
    /**
     * @description Returns true on [[Consensus]]
     */
    readonly isConsensus: boolean;
    /**
     * @description Returns true on [[Other]]
     */
    readonly isOther: boolean;
    /**
     * @description Returns true on [[PreRuntime]]
     */
    readonly isPreRuntime: boolean;
    /**
     * @description Returns true on [[Seal]]
     */
    readonly isSeal: boolean;
    /**
     * @description Returns true on [[SealV0]]
     */
    readonly isSealV0: boolean;
    /**
     * @description Converts the Object to JSON, typically used for RPC transfers. For logs, we overrides to produce the hex version (sligning with substrate gives in actual JSON responses)
     */
    toJSON(): string;
    /**
     * @description Returns the type of engine
     */
    readonly type: string;
}
/**
 * @name Digest
 * @description
 * A [[Header]] Digest
 */
export default class Digest extends Struct {
    constructor(value: any);
    /**
     * @description The [[DigestItem]] logs
     */
    readonly logs: Vec<DigestItem>;
    /**
     * @description The [[DigestItem]] logs, filtered, filter items included. This is useful for derive functionality where only a certain type of log is to be returned.
     */
    logsWith(...include: string[]): Vec<DigestItem>;
    /**
     * @description The [[DigestItem]] logs, filtered, filter items exluded. This is useful for stripping headers for eg. WASM runtime execution.
     */
    logsWithout(...exclude: string[]): Vec<DigestItem>;
}
